

#include <Servo.h>
Servo legServo[5];  // create servo object to control the 4 legs
Servo shouldServo[5];  // create servo object to control the 4 shoulders


const int FR = 0;
const int RR = 1;
const int FL = 2;
const int RL = 3;


int legRange = 30;
int shouldRange = 60;


int scnt = 0;
int serialRawData = 0;

int legHome[5] = {90,100,90,70};
int legMax[5];
int legMin[5];
int legDir[5];
int legDwell[5];

int shouldHome[5] = {100, 90, 80, 90};
int shouldMax[5];
int shouldMin[5];
int shouldDir[5];
int servoGo[5] = {1,0,0,0};

int legPos[5];
unsigned long a;
int shouldPos[5];

int led;

float legX[5],legY[5],shouldX[5],shouldY[5];
unsigned int legD[5], shouldD[5];
int liftShould[5];
int masterGear[5];
float w[5];
int saveMaster[5];


int dwellTicks = 100;
int mainCycleTicks = 1800 + 900 + dwellTicks;
int qtCycle = mainCycleTicks / 4;

int fullCycle = 0;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);
  pinMode(13,OUTPUT);
  Serial.println("LegFR,ShoulderFR,LegRR,ShoulderRR,LegFL,ShoulderFL,LegRL,ShoulderRLL");
  legServo[FR].attach(3);
  shouldServo[FR].attach(5);
  
  legServo[RR].attach(A5);
  shouldServo[RR].attach(A4);
  
  legServo[FL].attach(A1);
  shouldServo[FL].attach(A0);

  legServo[RL].attach(A3);
  shouldServo[RL].attach(A2);


  
  for (int u = 0; u <= 3; u++){
    legMax[u] = legHome[u] + legRange/2;
    legMin[u] = legHome[u] - legRange/2;
    shouldMax[u] = shouldHome[u] + shouldRange/2;
    shouldMin[u] = shouldHome[u] - shouldRange/2;
  }
  
  

  legPos[FR] = legMin[FR];
  legDir[FR] = -1;
  shouldPos[FR] = shouldMin[FR];
  shouldDir[FR] = -1;

  
  legPos[RR] = legMax[RR];
  legDir[RR] = -1;
  shouldPos[RR] = shouldMax[RR];
  shouldDir[RR] = 1;

  
  legPos[FL] = legMin[FL];
  legDir[FL] = 1;
  shouldPos[FL] = shouldMax[FL];
  shouldDir[FL] = 1;


  legPos[RL] = legMin[RL];
  legDir[RL] = 1;
  shouldPos[RL] = shouldMin[RL];
  shouldDir[RL] = -1;


  
  for (int pr = 0; pr <= 3; pr++){        
    legServo[pr].write(90);//legHome[pr]);
    shouldServo[pr].write(90);//shouldHome[pr]);
  }
 delay(3000);

}


void loop() {


  for (int b = 0; b <= 3; b++){


   if (legDwell[b] > 0){//dwell for an amount of time afte full leg cycle
    if (servoGo[b]) masterGear[b]++;
    if (masterGear[b] > saveMaster[b] + dwellTicks) {
      liftShould[b] = 1;
      legDwell[b] = 0;
    }
   }
   else{
     if (liftShould[b]){//lift the shoulder and speed up leg/shoulder movement.  Leg goes 180° and shoulder goes 360°
        if (servoGo[b]) masterGear[b]++;
        shouldD[b] += 4;
        legD[b] += 2;
        w[b] = (float)legD[b] / 1800 * PI;
        shouldY[b] = shouldRange / 2 * cos((float)shouldD[b] / 1800 * PI);    
        shouldPos[b] = shouldHome[b] + shouldDir[b] * (int) shouldY[b];
        legY[b] = legRange / 2 *  cos(w[b]);
        legPos[b] = legHome[b] + legDir[b] * legY[b];
        if (shouldD[b] >= 3600){
         shouldD[b] = 0;
         liftShould[b] = 0;
         legD[b] = 0;
         masterGear[b] = 0;
         if (b == 3) fullCycle = 1;
        }
      }
      else{//leg moving backwards as body move forward  Move leg through 180°
        if (servoGo[b]) {
          masterGear[b]++;
          legD[b] += 1;
        }
        
        w[b] = (float)legD[b] / 1800 * PI;
        legY[b] = legRange / 2 *  cos(w[b]);
        legPos[b] = legHome[b] + legDir[b] * legY[b];
        
        if (b < 3 && servoGo[b] && masterGear[b] > qtCycle) servoGo[b + 1] = 1;
        
        if (legD[b] > 1800){
          legDwell[b] = 1;
          saveMaster[b] = masterGear[b];

        }
      }

   }
  }
  
  scnt++;

  if (scnt > 20){
    scnt = 0;

      //use the serial plotter function to see a graph of the servo movements
      if (serialRawData){
        Serial.print(legD[FR]);
        
        Serial.print(",");

        Serial.print(legD[RR]);
        Serial.print(",");
        Serial.print(legD[FL]);
        Serial.print(",");
        Serial.print(legD[RL]);
        Serial.print(",");

        Serial.print(shouldD[FR]);
        Serial.print(",");   
        Serial.print(shouldD[RR]);
        Serial.print(",");   
        Serial.print(shouldD[FL]);
        Serial.print(",");
        Serial.print(shouldD[RL]);
        Serial.print(",");       
        
        Serial.print(masterGear[FR]);
        Serial.print(",");
        Serial.print(masterGear[RR]);
        Serial.print(",");
        Serial.print(masterGear[FL]);
        Serial.print(",");             
        Serial.print(masterGear[RL]);

        Serial.println();
      }
      else{//serial plot servo positions mapped to look better on the serial plotter output
        Serial.print(map(legPos[FR], legMin[FR], legMax[FR], 0, 60));
        Serial.print(",");
        Serial.print(map(shouldPos[FR], shouldMin[FR], shouldMax[FR], 0, 60));
        Serial.print(",");
  
        Serial.print(map(legPos[RR], legMin[RR], legMax[RR], 60, 120));
        Serial.print(",");
        Serial.print(map(shouldPos[RR], shouldMin[RR], shouldMax[RR], 60, 120));
        Serial.print(",");
        
        Serial.print(map(legPos[FL], legMin[FL], legMax[FL], 120, 180));
        Serial.print(",");
        Serial.print(map(shouldPos[FL], shouldMin[FL], shouldMax[FL], 120, 180));
        Serial.print(",");
  
  
        Serial.print(map(legPos[RL], legMin[RL], legMax[RL], 180, 240));
        Serial.print(",");
        Serial.print(map(shouldPos[RL], shouldMin[RL], shouldMax[RL], 180, 240));

       
        Serial.println();
      }      
     
      legServo[FR].write(legPos[FR]);
      shouldServo[FR].write(shouldPos[FR]);
      legServo[RR].write(legPos[RR]);
      shouldServo[RR].write(shouldPos[RR]);
      legServo[FL].write(legPos[FL]);
      shouldServo[FL].write(shouldPos[FL]);
      legServo[RL].write(legPos[RL]);
      shouldServo[RL].write(shouldPos[RL]);            
    
    digitalWrite(13, bitRead(led,5));
    led++;
    
  }
  if (fullCycle)
    delayMicroseconds(1000);

  
}
